@@ -240,6 +240,24 @@ describe "DB#create_table" do
     @db.sqls.must_equal ["CREATE TABLE cats (id integer, name text, UNIQUE (name) DEFERRABLE INITIALLY IMMEDIATE)"]
   end
   
+  it "should handle deferred unique column constraints" do
+    @db.create_table(:cats) do
+      integer :id, :unique=>true, :unique_deferrable=>true
+      integer :i, :unique=>true, :unique_deferrable=>:immediate
+      integer :j, :unique=>true, :unique_deferrable=>false
+    end
+    @db.sqls.must_equal ["CREATE TABLE cats (id integer UNIQUE DEFERRABLE INITIALLY DEFERRED, i integer UNIQUE DEFERRABLE INITIALLY IMMEDIATE, j integer UNIQUE NOT DEFERRABLE)"]
+  end
+  
+  it "should handle deferred primary key column constraints" do
+    @db.create_table(:cats) do
+      integer :id, :primary_key=>true, :primary_key_deferrable=>true
+      integer :i, :primary_key=>true, :primary_key_deferrable=>:immediate
+      integer :j, :primary_key=>true, :primary_key_deferrable=>false
+    end
+    @db.sqls.must_equal ["CREATE TABLE cats (id integer PRIMARY KEY DEFERRABLE INITIALLY DEFERRED, i integer PRIMARY KEY DEFERRABLE INITIALLY IMMEDIATE, j integer PRIMARY KEY NOT DEFERRABLE)"]
+  end
+  
   it "should accept unsigned definition" do
     @db.create_table(:cats) do
       integer :value, :unsigned => true
@@ -1777,17 +1795,21 @@ describe "Schema Parser" do
     @db.schema(:text).first.last[:type].must_equal :string
     @db.schema(:date).first.last[:type].must_equal :date
     @db.schema(:datetime).first.last[:type].must_equal :datetime
+    @db.schema(:smalldatetime).first.last[:type].must_equal :datetime
     @db.schema(:timestamp).first.last[:type].must_equal :datetime
     @db.schema(:"timestamp with time zone").first.last[:type].must_equal :datetime
     @db.schema(:"timestamp without time zone").first.last[:type].must_equal :datetime
     @db.schema(:time).first.last[:type].must_equal :time
     @db.schema(:"time with time zone").first.last[:type].must_equal :time
     @db.schema(:"time without time zone").first.last[:type].must_equal :time
+    @db.schema(:bool).first.last[:type].must_equal :boolean
     @db.schema(:boolean).first.last[:type].must_equal :boolean
     @db.schema(:real).first.last[:type].must_equal :float
     @db.schema(:float).first.last[:type].must_equal :float
+    @db.schema(:"float unsigned").first.last[:type].must_equal :float
     @db.schema(:double).first.last[:type].must_equal :float
     @db.schema(:"double(1,2)").first.last[:type].must_equal :float
+    @db.schema(:"double(1,2) unsigned").first.last[:type].must_equal :float
     @db.schema(:"double precision").first.last[:type].must_equal :float
     @db.schema(:number).first.last[:type].must_equal :decimal
     @db.schema(:numeric).first.last[:type].must_equal :decimal
@@ -1801,6 +1823,8 @@ describe "Schema Parser" do
     @db.schema(:nchar).first.last[:type].must_equal :string
     @db.schema(:nvarchar).first.last[:type].must_equal :string
     @db.schema(:ntext).first.last[:type].must_equal :string
+    @db.schema(:clob).first.last[:type].must_equal :string
+    @db.schema(:ntext).first.last[:type].must_equal :string
     @db.schema(:smalldatetime).first.last[:type].must_equal :datetime
     @db.schema(:binary).first.last[:type].must_equal :blob
     @db.schema(:varbinary).first.last[:type].must_equal :blob
