@@ -120,7 +120,7 @@ describe "Reversible Migrations with Sequel.migration{change{}}" do
       end
     end
     @db = @c.new
-    @p = Proc.new do
+    @p = proc do
       create_table(:a, :foo=>:bar){Integer :a}
       add_column :a, :b, String
       add_index :a, :b
@@ -195,6 +195,19 @@ describe "Reversible Migrations with Sequel.migration{change{}}" do
       [:drop_table, :a, {:foo=>:bar}]]
   end
   
+  it "should reverse add_foreign_key with :type option" do
+    Sequel.migration{change{alter_table(:t){add_foreign_key :b, :c, :type=>:f}}}.apply(@db, :down)
+    actions = @db.actions
+    actions.must_equal [[:alter_table, [[:drop_foreign_key, :b, {:type=>:f}]]]]
+    @db.sqls
+    db = Sequel.mock
+    args = nil
+    db.define_singleton_method(:foreign_key_list){|*a| args = a; [{:name=>:fbc, :columns=>[:b]}]}
+    db.alter_table(:t){send(*actions[0][1][0])}
+    db.sqls.must_equal ["ALTER TABLE t DROP CONSTRAINT fbc", "ALTER TABLE t DROP COLUMN b"]
+    args.must_equal [:t]
+  end
+  
   it "should reverse add_foreign_key with :foreign_key_constraint_name option" do
     Sequel.migration{change{alter_table(:t){add_foreign_key :b, :c, :foreign_key_constraint_name=>:f}}}.apply(@db, :down)
     actions = @db.actions
@@ -237,6 +250,11 @@ describe "Sequel::Migrator.migrator_class" do
     Sequel::IntegerMigrator.migrator_class("spec/files/timestamped_migrations").must_equal Sequel::IntegerMigrator
     Sequel::TimestampMigrator.migrator_class("spec/files/integer_migrations").must_equal Sequel::TimestampMigrator
   end
+
+  it "should raise an error if the migration folder does not exist" do
+    proc{Sequel::Migrator.apply(@db, "spec/files/nonexistant_migration_path")}.must_raise(Sequel::Migrator::Error)
+  end
+  
 end
 
 describe "Sequel::IntegerMigrator" do
@@ -283,19 +301,19 @@ describe "Sequel::IntegerMigrator" do
   after do
     Object.send(:remove_const, "CreateSessions") if Object.const_defined?("CreateSessions")
   end
-  
-  it "should raise and error if there is a missing integer migration version" do
+
+  it "should raise an error if there is a missing integer migration version" do
     proc{Sequel::Migrator.apply(@db, "spec/files/missing_integer_migrations")}.must_raise(Sequel::Migrator::Error)
   end
   
-  it "should not raise and error if there is a missing integer migration version and allow_missing_migration_files is true" do
+  it "should not raise an error if there is a missing integer migration version and allow_missing_migration_files is true" do
     Sequel::Migrator.run(@db, "spec/files/missing_integer_migrations", :allow_missing_migration_files => true)
     @db.sqls.last.must_equal "UPDATE schema_info SET version = 3"
     Sequel::Migrator.run(@db, "spec/files/missing_integer_migrations", :allow_missing_migration_files => true, :target=>0)
     @db.sqls.last.must_equal "UPDATE schema_info SET version = 0"
   end
 
-  it "should raise and error if there is a duplicate integer migration version" do
+  it "should raise an error if there is a duplicate integer migration version" do
     proc{Sequel::Migrator.apply(@db, "spec/files/duplicate_integer_migrations")}.must_raise(Sequel::Migrator::Error)
   end
 
@@ -307,6 +325,12 @@ describe "Sequel::IntegerMigrator" do
     proc{Sequel::Migrator.apply(@db, "spec/files/double_migration")}.must_raise(Sequel::Migrator::Error)
   end
 
+  it "should raise an error if the most recent migration can't be detected" do
+    # Have to specify a target version, otherwise an earlier check (inability
+    # to detect the target) would raise an error, falsely matching the check.
+    proc{Sequel::Migrator.apply(@db, "spec/files/empty_migration_folder", 2)}.must_raise(Sequel::Migrator::Error)
+  end
+
   it "should add a column name if it doesn't already exist in the schema_info table" do
     @db.create_table(:schema_info){Integer :v}
     def @db.alter_table(*); end
