@@ -49,6 +49,12 @@ describe "NestedAttributes plugin" do
     @db.sqls
   end
   
+  it "should not modify options hash when loading plugin" do
+    h = {}
+    @Concert.nested_attributes :albums, h
+    h.must_equal({})
+  end
+  
   it "should support creating new many_to_one objects" do
     a = @Album.new({:name=>'Al', :artist_attributes=>{:name=>'Ar'}})
     @db.sqls.must_equal []
@@ -486,6 +492,112 @@ describe "NestedAttributes plugin" do
     @db.sqls.must_equal ["UPDATE artists SET name = 'Ar' WHERE (id = 10)"]
   end
 
+  it "should raise a NoExistingObject error if object to be updated no longer exists, if the :require_modification=>true option is used" do
+    @Artist.nested_attributes :albums, :require_modification=>true, :destroy=>true
+    al = @Album.load(:id=>10, :name=>'Al')
+    ar = @Artist.load(:id=>20, :name=>'Ar')
+    ar.associations[:albums] = [al]
+    ar.set(:albums_attributes=>[{:id=>10, :name=>'L'}])
+    @db.sqls.must_equal []
+    @db.numrows = [1, 0]
+    proc{ar.save}.must_raise Sequel::NoExistingObject
+    @db.sqls.must_equal ["UPDATE artists SET name = 'Ar' WHERE (id = 20)", "UPDATE albums SET name = 'L' WHERE (id = 10)"]
+  end
+
+  it "should not raise an Error if object to be updated no longer exists, if the :require_modification=>false option is used" do
+    @Artist.nested_attributes :albums, :require_modification=>false, :destroy=>true
+    al = @Album.load(:id=>10, :name=>'Al')
+    ar = @Artist.load(:id=>20, :name=>'Ar')
+    ar.associations[:albums] = [al]
+    ar.set(:albums_attributes=>[{:id=>10, :name=>'L'}])
+    @db.sqls.must_equal []
+    @db.numrows = [1, 0]
+    ar.save
+    @db.sqls.must_equal ["UPDATE artists SET name = 'Ar' WHERE (id = 20)", "UPDATE albums SET name = 'L' WHERE (id = 10)"]
+  end
+
+  it "should raise a NoExistingObject error if object to be deleted no longer exists, if the :require_modification=>true option is used" do
+    @Artist.nested_attributes :albums, :require_modification=>true, :destroy=>true
+    al = @Album.load(:id=>10, :name=>'Al')
+    ar = @Artist.load(:id=>20, :name=>'Ar')
+    ar.associations[:albums] = [al]
+    ar.set(:albums_attributes=>[{:id=>10, :_delete=>'t'}])
+    @db.sqls.must_equal []
+    @db.numrows = [1, 0]
+    proc{ar.save}.must_raise Sequel::NoExistingObject
+    @db.sqls.must_equal ["UPDATE artists SET name = 'Ar' WHERE (id = 20)", "DELETE FROM albums WHERE (id = 10)"]
+  end
+
+  it "should not raise an Error if object to be deleted no longer exists, if the :require_modification=>false option is used" do
+    @Artist.nested_attributes :albums, :require_modification=>false, :destroy=>true
+    al = @Album.load(:id=>10, :name=>'Al')
+    ar = @Artist.load(:id=>20, :name=>'Ar')
+    ar.associations[:albums] = [al]
+    ar.set(:albums_attributes=>[{:id=>10, :_delete=>'t'}])
+    @db.sqls.must_equal []
+    @db.numrows = [1, 0]
+    ar.save
+    @db.sqls.must_equal ["UPDATE artists SET name = 'Ar' WHERE (id = 20)", "DELETE FROM albums WHERE (id = 10)"]
+  end
+
+  it "should not attempt to validate nested attributes twice for one_to_many associations when creating them" do
+    @Artist.nested_attributes :albums
+    validated = []
+    @Album.class_eval do
+      define_method(:validate) do
+        super()
+        validated << self
+      end
+    end
+    a = @Artist.new(:name=>'Ar', :albums_attributes=>[{:name=>'Al'}])
+    @db.sqls.must_equal []
+    validated.length.must_equal 0
+    a.save
+    validated.length.must_equal 1
+    check_sql_array("INSERT INTO artists (name) VALUES ('Ar')",
+      ["INSERT INTO albums (artist_id, name) VALUES (1, 'Al')", "INSERT INTO albums (name, artist_id) VALUES ('Al', 1)"])
+  end
+  
+  it "should not attempt to validate nested attributes twice for one_to_one associations when creating them" do
+    @Artist.nested_attributes :first_album
+    validated = []
+    @Album.class_eval do
+      define_method(:validate) do
+        super()
+        validated << self
+      end
+    end
+    a = @Artist.new(:name=>'Ar', :first_album_attributes=>{:name=>'Al'})
+    @db.sqls.must_equal []
+    validated.length.must_equal 0
+    a.save
+    validated.length.must_equal 1
+    check_sql_array("INSERT INTO artists (name) VALUES ('Ar')",
+      "UPDATE albums SET artist_id = NULL WHERE (artist_id = 1)",
+      "INSERT INTO albums (name, artist_id) VALUES ('Al', 1)")
+  end
+  
+  it "should not clear reciprocal association before saving new one_to_one associated object" do
+    @Artist.one_to_one :first_album, :clone=>:first_album, :reciprocal=>:artist
+    @Artist.nested_attributes :first_album
+    assoc = []
+    @Album.class_eval do
+      define_method(:after_save) do
+        super()
+        assoc << associations[:artist]
+      end
+    end
+    a = @Artist.new(:name=>'Ar', :first_album_attributes=>{:name=>'Al'})
+    @db.sqls.must_equal []
+    assoc.must_be_empty
+    a.save
+    assoc.length.must_equal 1
+    assoc.first.must_be_kind_of(@Artist)
+    check_sql_array("INSERT INTO artists (name) VALUES ('Ar')",
+      "UPDATE albums SET artist_id = NULL WHERE (artist_id = 1)",
+      "INSERT INTO albums (name, artist_id) VALUES ('Al', 1)")
+  end
+  
   it "should not save if nested attribute is not valid and should include nested attribute validation errors in the main object's validation errors" do
     @Artist.class_eval do
       def validate
