@@ -1,22 +1,44 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
-
+require_relative "spec_helper"
 
 describe "pg_range extension" do
   before(:all) do
     Sequel.extension :pg_array, :pg_range
-    @pg_types = Sequel::Postgres::PG_TYPES.dup
-  end
-  after(:all) do
-    Sequel::Postgres::PG_TYPES.replace(@pg_types)
   end
 
   before do
-    @db = Sequel.connect('mock://postgres', :quote_identifiers=>false)
+    @db = Sequel.connect('mock://postgres')
     @R = Sequel::Postgres::PGRange
-    @db.extend_datasets(Module.new{def supports_timestamp_timezones?; false; end; def supports_timestamp_usecs?; false; end})
+    @db.extend_datasets do
+      def supports_timestamp_timezones?; false end
+      def supports_timestamp_usecs?; false end
+      def quote_identifiers?; false end
+    end
     @db.extension(:pg_array, :pg_range)
   end
 
+  endless_range_support = RUBY_VERSION >= '2.6'
+  startless_range_support = RUBY_VERSION >= '2.7'
+
+  it "should set up conversion procs correctly" do
+    cp = @db.conversion_procs
+    cp[3904].call("[1,2]").must_equal @R.new(1,2, :exclude_begin=>false, :exclude_end=>false, :db_type=>'int4range')
+    cp[3906].call("[1,2]").must_equal @R.new(1,2, :exclude_begin=>false, :exclude_end=>false, :db_type=>'numrange')
+    cp[3908].call("[2011-01-02 10:20:30,2011-02-03 10:20:30)").must_equal @R.new(Time.local(2011, 1, 2, 10, 20, 30),Time.local(2011, 2, 3, 10, 20, 30), :exclude_begin=>false, :exclude_end=>true, :db_type=>'tsrange')
+    cp[3910].call("[2011-01-02 10:20:30,2011-02-03 10:20:30)").must_equal @R.new(Time.local(2011, 1, 2, 10, 20, 30),Time.local(2011, 2, 3, 10, 20, 30), :exclude_begin=>false, :exclude_end=>true, :db_type=>'tstzrange')
+    cp[3912].call("[2011-01-02,2011-02-03)").must_equal  @R.new(Date.new(2011, 1, 2),Date.new(2011, 2, 3), :exclude_begin=>false, :exclude_end=>true, :db_type=>'daterange')
+    cp[3926].call("[1,2]").must_equal @R.new(1,2, :exclude_begin=>false, :exclude_end=>false, :db_type=>'int8range')
+  end
+
+  it "should set up conversion procs for arrays correctly" do
+    cp = @db.conversion_procs
+    cp[3905].call("{\"[1,2]\"}").must_equal [@R.new(1,2, :exclude_begin=>false, :exclude_end=>false, :db_type=>'int4range')]
+    cp[3907].call("{\"[1,2]\"}").must_equal [@R.new(1,2, :exclude_begin=>false, :exclude_end=>false, :db_type=>'numrange')]
+    cp[3909].call("{\"[2011-01-02 10:20:30,2011-02-03 10:20:30)\"}").must_equal [@R.new(Time.local(2011, 1, 2, 10, 20, 30),Time.local(2011, 2, 3, 10, 20, 30), :exclude_begin=>false, :exclude_end=>true, :db_type=>'tsrange')]
+    cp[3911].call("{\"[2011-01-02 10:20:30,2011-02-03 10:20:30)\"}").must_equal [@R.new(Time.local(2011, 1, 2, 10, 20, 30),Time.local(2011, 2, 3, 10, 20, 30), :exclude_begin=>false, :exclude_end=>true, :db_type=>'tstzrange')]
+    cp[3913].call("{\"[2011-01-02,2011-02-03)\"}").must_equal [@R.new(Date.new(2011, 1, 2),Date.new(2011, 2, 3), :exclude_begin=>false, :exclude_end=>true, :db_type=>'daterange')]
+    cp[3927].call("{\"[1,2]\"}").must_equal [@R.new(1,2, :exclude_begin=>false, :exclude_end=>false, :db_type=>'int8range')]
+  end
+
   it "should literalize Range instances to strings correctly" do
     @db.literal(Date.new(2011, 1, 2)...Date.new(2011, 3, 2)).must_equal "'[2011-01-02,2011-03-02)'"
     @db.literal(Time.local(2011, 1, 2, 10, 20, 30)...Time.local(2011, 2, 3, 10, 20, 30)).must_equal "'[2011-01-02 10:20:30,2011-02-03 10:20:30)'"
@@ -24,11 +46,26 @@ describe "pg_range extension" do
     @db.literal(DateTime.new(2011, 1, 2, 10, 20, 30)...DateTime.new(2011, 2, 3, 10, 20, 30)).must_equal "'[2011-01-02 10:20:30,2011-02-03 10:20:30)'"
     @db.literal(1..2).must_equal "'[1,2]'"
     @db.literal(1.0..2.0).must_equal "'[1.0,2.0]'"
-    @db.literal(BigDecimal.new('1.0')..BigDecimal.new('2.0')).must_equal "'[1.0,2.0]'"
+    @db.literal(BigDecimal('1.0')..BigDecimal('2.0')).must_equal "'[1.0,2.0]'"
     @db.literal(Sequel.lit('a')..Sequel.lit('z')).must_equal "'[a,z]'"
     @db.literal(''..'()[]",\\2').must_equal "'[\"\",\\(\\)\\[\\]\\\"\\,\\\\2]'"
   end
 
+  it "should literalize endless Range instances to strings correctly" do
+    @db.literal(eval('(1..)')).must_equal "'[1,]'"
+    @db.literal(eval('(1...)')).must_equal "'[1,)'"
+  end if endless_range_support
+
+  it "should literalize startless Range instances to strings correctly" do
+    @db.literal(eval('(..1)')).must_equal "'[,1]'"
+    @db.literal(eval('(...1)')).must_equal "'[,1)'"
+  end if startless_range_support
+
+  it "should literalize startless, endless Range instances to strings correctly" do
+    @db.literal(eval('nil..nil')).must_equal "'[,]'"
+    @db.literal(eval('nil...nil')).must_equal "'[,)'"
+  end if startless_range_support
+
   it "should literalize PGRange instances to strings correctly" do
     @db.literal(@R.new(1, 2)).must_equal "'[1,2]'"
     @db.literal(@R.new(true, false)).must_equal "'[true,false]'"
@@ -52,6 +89,21 @@ describe "pg_range extension" do
     @db.bound_variable_arg(1..2, nil).must_equal "[1,2]"
   end
 
+  it "should support using endless Range instances as bound variables" do
+    @db.bound_variable_arg(eval('(1..)'), nil).must_equal "[1,]"
+    @db.bound_variable_arg(eval('(1...)'), nil).must_equal "[1,)"
+  end if endless_range_support
+
+  it "should support using startless Range instances as bound variables" do
+    @db.bound_variable_arg(eval('(..1)'), nil).must_equal "[,1]"
+    @db.bound_variable_arg(eval('(...1)'), nil).must_equal "[,1)"
+  end if startless_range_support
+
+  it "should support using startless, endless Range instances as bound variables" do
+    @db.bound_variable_arg(eval('nil..nil'), nil).must_equal "[,]"
+    @db.bound_variable_arg(eval('nil...nil'), nil).must_equal "[,)"
+  end if startless_range_support
+
   it "should support using PGRange instances as bound variables" do
     @db.bound_variable_arg(@R.new(1, 2), nil).must_equal "[1,2]"
   end
@@ -60,6 +112,10 @@ describe "pg_range extension" do
     @db.bound_variable_arg([1..2,2...3], nil).must_equal '{"[1,2]","[2,3)"}'
   end
 
+  it "should support using arrays of endless Range instances as bound variables" do
+    @db.bound_variable_arg([eval('(1..)'), eval('(2..)')], nil).must_equal '{"[1,]","[2,]"}'
+  end if endless_range_support
+
   it "should support using PGRange instances as bound variables" do
     @db.bound_variable_arg([@R.new(1, 2),@R.new(2, 3)], nil).must_equal '{"[1,2]","[2,3]"}'
   end
@@ -74,6 +130,26 @@ describe "pg_range extension" do
     @db.schema(:items).map{|e| e[1][:type]}.must_equal [:integer, :int4range_array, :int8range_array, :numrange_array, :daterange_array, :tsrange_array, :tstzrange_array]
   end
 
+  it "should set :ruby_default schema entries if default value is recognized" do
+    @db.fetch = [{:name=>'id', :db_type=>'integer', :default=>'1'}, {:oid=>3904, :name=>'t', :db_type=>'int4range', :default=>"'[1,5)'::int4range"}]
+    s = @db.schema(:items)
+    s[1][1][:ruby_default].must_equal Sequel::Postgres::PGRange.new(1, 5, :exclude_end=>true, :db_type=>'int4range')
+  end
+
+  it "should work correctly in hashes" do
+    h = Hash.new(1)
+    h[@R.new(1, 2)] = 2
+    h[@R.new(nil, nil, :empty => true)] = 3
+    h[@R.new(1, 2)].must_equal 2
+    h[@R.new(1, 3)].must_equal 1
+    h[@R.new(2, 2)].must_equal 1
+    h[@R.new(1, 2, :exclude_begin => true)].must_equal 1
+    h[@R.new(1, 2, :exclude_end => true)].must_equal 1
+    h[@R.new(1, 2, :db_type => :int)].must_equal 1
+    h[@R.new(nil, nil, :empty => true)].must_equal 3
+    h[@R.new(nil, nil, :empty => true, :db_type => :int)].must_equal 1
+  end
+
   describe "database typecasting" do
     before do
       @o = @R.new(1, 2, :db_type=>'int4range')
@@ -128,42 +204,47 @@ describe "pg_range extension" do
   end
 
   it "should support registering custom range types" do
-    @R.register('foorange')
+    @db.register_range_type('foorange')
     @db.typecast_value(:foorange, 1..2).must_be_kind_of(@R)
     @db.fetch = [{:name=>'id', :db_type=>'foorange'}]
     @db.schema(:items).map{|e| e[1][:type]}.must_equal [:foorange]
   end
 
   it "should support using a block as a custom conversion proc given as block" do
-    @R.register('foo2range'){|s| (s*2).to_i}
+    @db.register_range_type('foo2range'){|s| (s*2).to_i}
     @db.typecast_value(:foo2range, '[1,2]').must_be :==, (11..22)
   end
 
   it "should support using a block as a custom conversion proc given as :converter option" do
-    @R.register('foo3range', :converter=>proc{|s| (s*2).to_i})
+    @db.register_range_type('foo3range', :converter=>proc{|s| (s*2).to_i})
     @db.typecast_value(:foo3range, '[1,2]').must_be :==, (11..22)
   end
 
   it "should support using an existing scaler conversion proc via the :subtype_oid option" do
-    @R.register('foo4range', :subtype_oid=>16)
+    @db.register_range_type('foo4range', :subtype_oid=>16)
     @db.typecast_value(:foo4range, '[t,f]').must_equal @R.new(true, false, :db_type=>'foo4range')
   end
 
   it "should raise an error if using :subtype_oid option with unexisting scalar conversion proc" do
-    proc{@R.register('fooirange', :subtype_oid=>0)}.must_raise(Sequel::Error)
+    proc{@db.register_range_type('fooirange', :subtype_oid=>0)}.must_raise(Sequel::Error)
   end
 
   it "should raise an error if using :converter option and a block argument" do
-    proc{@R.register('fooirange', :converter=>proc{}){}}.must_raise(Sequel::Error)
+    proc{@db.register_range_type('fooirange', :converter=>proc{}){}}.must_raise(Sequel::Error)
   end
 
   it "should raise an error if using :subtype_oid option and a block argument" do
-    proc{@R.register('fooirange', :subtype_oid=>16){}}.must_raise(Sequel::Error)
+    proc{@db.register_range_type('fooirange', :subtype_oid=>16){}}.must_raise(Sequel::Error)
   end
 
   it "should support registering custom types with :oid option" do
-    @R.register('foo5range', :oid=>331)
-    Sequel::Postgres::PG_TYPES[331].call('[1,3)').must_be_kind_of(@R)
+    @db.register_range_type('foo5range', :oid=>331)
+    @db.conversion_procs[331].call('[1,3)').must_be_kind_of(@R)
+  end
+
+  it "should not support registering custom range types on a per-Database basis for frozen databases" do
+    @db.freeze
+    proc{@db.register_range_type('banana', :oid=>7865){|s| s}}.must_raise RuntimeError, TypeError
   end
 
   it "should support registering custom range types on a per-Database basis" do
@@ -213,7 +294,7 @@ describe "pg_range extension" do
 
   describe "parser" do
     before do
-      @p = Sequel::Postgres::PG_TYPES[3904]
+      @p = @R::Parser.new('int4range', proc(&:to_i))
       @sp = @R::Parser.new(nil)
     end
 
@@ -256,20 +337,6 @@ describe "pg_range extension" do
     end
   end
 
-  it "should set appropriate timestamp range conversion procs when resetting conversion procs" do
-    @db.reset_conversion_procs
-    procs = @db.conversion_procs
-    procs[3908].call('[2011-10-20 11:12:13,2011-10-20 11:12:14]').must_be :==, (Time.local(2011, 10, 20, 11, 12, 13)..(Time.local(2011, 10, 20, 11, 12, 14)))
-    procs[3910].call('[2011-10-20 11:12:13,2011-10-20 11:12:14]').must_be :==, (Time.local(2011, 10, 20, 11, 12, 13)..(Time.local(2011, 10, 20, 11, 12, 14)))
-  end
-
-  it "should set appropriate timestamp range array conversion procs when resetting conversion procs" do
-    @db.reset_conversion_procs
-    procs = @db.conversion_procs
-    procs[3909].call('{"[2011-10-20 11:12:13,2011-10-20 11:12:14]"}').must_be :==, [Time.local(2011, 10, 20, 11, 12, 13)..Time.local(2011, 10, 20, 11, 12, 14)]
-    procs[3911].call('{"[2011-10-20 11:12:13,2011-10-20 11:12:14]"}').must_be :==, [Time.local(2011, 10, 20, 11, 12, 13)..Time.local(2011, 10, 20, 11, 12, 14)]
-  end
-
   describe "a PGRange instance" do
     before do
       @r1 = @R.new(1, 2)
@@ -280,17 +347,17 @@ describe "pg_range extension" do
     it "should have #begin return the beginning of the range" do
       @r1.begin.must_equal 1
       @r2.begin.must_equal 3
-      @r3.begin.must_equal nil
+      @r3.begin.must_be_nil
     end
 
     it "should have #end return the end of the range" do
       @r1.end.must_equal 2
-      @r2.end.must_equal nil
+      @r2.end.must_be_nil
       @r3.end.must_equal 4
     end
 
     it "should have #db_type return the range's database type" do
-      @r1.db_type.must_equal nil
+      @r1.db_type.must_be_nil
       @r2.db_type.must_equal 'int4range'
       @r3.db_type.must_equal 'int8range'
     end
@@ -327,10 +394,8 @@ describe "pg_range extension" do
     it "should quack like a range" do
       @r1.cover?(1.5).must_equal true
       @r1.cover?(2.5).must_equal false
-      if RUBY_VERSION >= '1.9'
-        @r1.first(1).must_equal [1]
-        @r1.last(1).must_equal [2]
-      end
+      @r1.first(1).must_equal [1]
+      @r1.last(1).must_equal [2]
       @r1.to_a.must_equal [1, 2]
       @r1.first.must_equal 1
       @r1.last.must_equal 2
@@ -378,12 +443,12 @@ describe "pg_range extension" do
       @R.new(nil, nil).wont_equal @R.new(nil, nil, :empty=>true)
     end
 
-    it "should only consider empty PGRanges equal if they have the same bounds" do
+    it "should only consider PGRanges equal if they have the same bounds" do
       @R.new(1, 2).must_equal @R.new(1, 2)
       @R.new(1, 2).wont_equal @R.new(1, 3)
     end
 
-    it "should only consider empty PGRanges equal if they have the same bound exclusions" do
+    it "should only consider PGRanges equal if they have the same bound exclusions" do
       @R.new(1, 2, :exclude_begin=>true).must_equal @R.new(1, 2, :exclude_begin=>true)
       @R.new(1, 2, :exclude_end=>true).must_equal @R.new(1, 2, :exclude_end=>true)
       @R.new(1, 2, :exclude_begin=>true).wont_equal @R.new(1, 2, :exclude_end=>true)
@@ -399,8 +464,39 @@ describe "pg_range extension" do
 
     it "should not consider a PGRange equal with a Range if it can't be expressed as a range" do
       @R.new(nil, nil).wont_be :==, (1..2)
+      if startless_range_support
+        @R.new(nil, nil, :exclude_begin=>true).wont_be :==, eval('nil..nil')
+      end
     end
 
+    it "should consider PGRanges equal with a endless Range they represent" do
+      @R.new(1, nil).must_be :==, eval('(1..)')
+      @R.new(1, nil, :exclude_end=>true).must_be :==, eval('(1...)')
+      @R.new(1, nil).wont_be :==, eval('(1...)')
+      @R.new(1, nil, :exclude_end=>true).wont_be :==, eval('(1..)')
+      @R.new(1, nil).wont_be :==, eval('(2..)')
+      @R.new(1, nil, :exclude_end=>true).wont_be :==, eval('(2...)')
+    end if endless_range_support
+
+    it "should consider PGRanges equal with a startless Range they represent" do
+      @R.new(nil, 1).must_be :==, eval('(..1)')
+      @R.new(nil, 1, :exclude_end=>true).must_be :==, eval('(...1)')
+      @R.new(nil, 1).wont_be :==, eval('(...1)')
+      @R.new(nil, 1, :exclude_end=>true).wont_be :==, eval('(..1)')
+      @R.new(nil, 1).wont_be :==, eval('(..2)')
+      @R.new(nil, 1, :exclude_end=>true).wont_be :==, eval('(...2)')
+    end if startless_range_support
+
+    it "should consider PGRanges equal with a startless, endless Range they represent" do
+      @R.new(nil, nil).must_be :==, eval('nil..nil')
+      @R.new(nil, nil, :exclude_end=>true).must_be :==, eval('nil...nil')
+      @R.new(nil, nil).wont_be :==, eval('nil...nil')
+      @R.new(nil, nil, :exclude_end=>true).wont_be :==, eval('nil..nil')
+      @R.new(nil, nil).wont_be :==, eval('nil..1')
+      @R.new(nil, nil).wont_be :==, eval('1..nil')
+      @R.new(1, nil).wont_be :==, eval('nil..nil')
+    end if startless_range_support
+
     it "should not consider a PGRange equal to other objects" do
       @R.new(nil, nil).wont_equal 1
     end
@@ -408,7 +504,6 @@ describe "pg_range extension" do
     it "should have #=== be true if given an equal PGRange" do
       @R.new(1, 2).must_be :===, @R.new(1, 2)
       @R.new(1, 2).wont_be :===, @R.new(1, 3)
-
     end
 
     it "should have #=== be true if it would be true for the Range represented by the PGRange" do
@@ -417,7 +512,7 @@ describe "pg_range extension" do
     end
 
     it "should have #=== be false if the PGRange cannot be represented by a Range" do
-      @R.new(nil, nil).wont_be :===, 1.5
+      @R.new(1, 2, :exclude_begin=>true).wont_be :===, 1.5
     end
 
     it "should have #empty? indicate whether the range is empty" do
@@ -435,8 +530,6 @@ describe "pg_range extension" do
     end
 
     it "should have #to_range raise an exception if the PGRange cannot be represented by a Range" do
-      proc{@R.new(nil, 1).to_range}.must_raise(Sequel::Error)
-      proc{@R.new(1, nil).to_range}.must_raise(Sequel::Error)
       proc{@R.new(0, 1, :exclude_begin=>true).to_range}.must_raise(Sequel::Error)
       proc{@R.empty.to_range}.must_raise(Sequel::Error)
     end
@@ -445,6 +538,30 @@ describe "pg_range extension" do
       @r1.to_range.must_be :==, (1..2)
     end
 
+    it "should have #to_range return the represented range for endless ranges" do
+      @R.new(1, nil).to_range.must_be :==, eval('1..')
+    end if endless_range_support
+
+    it "should have #to_range raise an exception for endless ranges" do
+      proc{@R.new(1, nil).to_range}.must_raise(Sequel::Error)
+    end unless endless_range_support
+
+    it "should have #to_range return the represented range for startless ranges" do
+      @R.new(nil, 1).to_range.must_be :==, eval('..1')
+    end if startless_range_support
+
+    it "should have #to_range raise an exception for startless ranges" do
+      proc{@R.new(nil, 1).to_range}.must_raise(Sequel::Error)
+    end unless startless_range_support
+
+    it "should have #to_range return the represented range for startless, endless ranges" do
+      @R.new(nil, nil).to_range.must_be :==, eval('nil..nil')
+    end if startless_range_support
+
+    it "should have #to_range raise an exception for startless, endless ranges" do
+      proc{@R.new(nil, nil).to_range}.must_raise(Sequel::Error)
+    end unless startless_range_support
+
     it "should have #to_range cache the returned value" do
       @r1.to_range.must_be_same_as(@r1.to_range)
     end
@@ -464,10 +581,20 @@ describe "pg_range extension" do
     end
 
     it "should have #valid_ruby_range? return false if the PGRange cannot be represented as a Range" do
-      @R.new(nil, 1).valid_ruby_range?.must_equal false
-      @R.new(1, nil).valid_ruby_range?.must_equal false
       @R.new(0, 1, :exclude_begin=>true).valid_ruby_range?.must_equal false
       @R.empty.valid_ruby_range?.must_equal false
     end
+
+   it "should have #valid_ruby_range return #{endless_range_support} for endless ranges" do
+      @R.new(1, nil).valid_ruby_range?.must_equal(endless_range_support)
+    end
+
+   it "should have #valid_ruby_range return #{startless_range_support} for endless ranges" do
+      @R.new(nil, 1).valid_ruby_range?.must_equal(startless_range_support)
+    end
+
+   it "should have #valid_ruby_range return #{startless_range_support} for startless, endless ranges" do
+      @R.new(nil, nil).valid_ruby_range?.must_equal(startless_range_support)
+    end
   end
 end
