@@ -39,6 +39,15 @@ describe "Simple Dataset operations" do
     @ds.order(:id).all.must_equal [{:id=>1, :number=>10}, {:id=>100, :number=>20}]
   end
 
+  it "should support ordering considering NULLS" do
+    @ds.insert(:number=>20)
+    @ds.insert(:number=>nil)
+    @ds.order(Sequel[:number].asc(:nulls=>:first)).select_map(:number).must_equal [nil, 10, 20]
+    @ds.order(Sequel[:number].asc(:nulls=>:last)).select_map(:number).must_equal [10, 20, nil]
+    @ds.order(Sequel[:number].desc(:nulls=>:first)).select_map(:number).must_equal [nil, 20, 10]
+    @ds.order(Sequel[:number].desc(:nulls=>:last)).select_map(:number).must_equal [20, 10, nil]
+  end
+
   it "should have insert return primary key value" do
     @ds.insert(:number=>20).must_equal 2
     @ds.filter(:id=>2).first[:number].must_equal 20
@@ -83,6 +92,17 @@ describe "Simple Dataset operations" do
     @ds.from_self(:alias=>:items).graph(@ds.from_self, {:id=>:id}, :table_alias=>:b).extension(:graph_each).all.must_equal [{:items=>{:id=>1, :number=>10}, :b=>{:id=>1, :number=>10}}]
   end
 
+  cspecify "should have insert and update work with Sequel::DEFAULT", :sqlite do
+    @db.create_table!(:items) do
+      Integer :number, :default=>10
+    end
+    @ds.insert(:number=>Sequel::DEFAULT)
+    @ds.select_map(:number).must_equal [10]
+    @ds.insert(:number=>20)
+    @ds.update(:number=>Sequel::DEFAULT)
+    @ds.select_map(:number).must_equal [10, 10]
+  end
+
   cspecify "should have insert work correctly when inserting a row with all NULL values", :hsqldb do
     @db.create_table!(:items) do
       String :name
@@ -140,6 +160,10 @@ describe "Simple Dataset operations" do
       rows.must_equal((1..100).map{|i| {:id=>i, :number=>i*10}}.reverse)
     end
 
+    rows = []
+    @ds.order(:number).limit(50, 25).paged_each(:rows_per_fetch=>3).each{|row| rows << row}
+    rows.must_equal((26..75).map{|i| {:id=>i, :number=>i*10}})
+
     rows = []
     @ds.order(:number).limit(50, 25).paged_each(:rows_per_fetch=>3){|row| rows << row}
     rows.must_equal((26..75).map{|i| {:id=>i, :number=>i*10}})
@@ -183,6 +207,24 @@ describe "Simple Dataset operations" do
     end
   end if DB.dataset.supports_skip_locked?
   
+  it "should raise error instead of waiting for rows correctly" do
+    @ds.insert(:number=>10)
+    q1 = Queue.new
+    q2 = Queue.new
+    ds = @ds.order(:id).for_update.nowait
+    begin
+      t = Thread.new{@db.transaction(:isolation=>:committed){q2.push(ds.get(:id)); q1.pop}}
+      q2.pop.must_equal 1
+      # Some databases do row level locking, others do page level locking
+      proc{@db.transaction(:isolation=>:committed){ds.get(:id)}}.must_raise Sequel::DatabaseLockTimeout
+    ensure
+      q1.push(nil)
+      t.join
+      # Keep only one active connection, as some other specs expect that
+      @db.disconnect
+    end
+  end if DB.dataset.supports_nowait?
+  
   it "should raise exception if raising on duplication columns" do
     proc{@ds.select_map([:id, :id])}.must_raise Sequel::DuplicateColumnError
   end if DB.opts[:on_duplicate_columns] == :raise
@@ -412,6 +454,20 @@ describe Sequel::Dataset do
     @d.extension(:sequel_4_dataset_methods).interval(:value).to_i.must_equal 6
   end
 
+  it "should support or emulate filtered aggregate functions" do
+    @d.insert(:name => 'abc', :value => 123)
+    @d.insert(:name => 'abc', :value => 456)
+    @d.insert(:name => 'def', :value => 324)
+    @d.get{count.function.*.filter{value > 100}}.must_equal 3
+    @d.get{count.function.*.filter{value > 200}}.must_equal 2
+    @d.get{count.function.*.filter{value > 400}}.must_equal 1
+    @d.get{count.function.*.filter{value > 500}}.must_equal 0
+    @d.get{count(:value).filter{value > 100}}.must_equal 3
+    @d.get{count(:value).filter{value > 200}}.must_equal 2
+    @d.get{count(:value).filter{value > 400}}.must_equal 1
+    @d.get{count(:value).filter{value > 500}}.must_equal 0
+  end
+
   it "should return the correct records" do
     @d.to_a.must_equal []
     @d.insert(:name => 'abc', :value => 123)
@@ -537,22 +593,21 @@ describe Sequel::Dataset do
 end
 
 describe "Simple Dataset operations" do
-  before do
+  before(:all) do
     DB.create_table!(:items) do
       Integer :number
       TrueClass :flag
     end
-    @ds = DB[:items]
+    @ds = DB[:items].order(:number)
+    @ds.insert(:number=>1, :flag=>true)
+    @ds.insert(:number=>2, :flag=>false)
+    @ds.insert(:number=>3, :flag=>nil)
   end
-  after do
+  after(:all) do
     DB.drop_table?(:items)
   end
 
   it "should deal with boolean conditions correctly" do
-    @ds.insert(:number=>1, :flag=>true)
-    @ds.insert(:number=>2, :flag=>false)
-    @ds.insert(:number=>3, :flag=>nil)
-    @ds = @ds.order(:number)
     @ds.filter(:flag=>true).map(:number).must_equal [1]
     @ds.filter(:flag=>false).map(:number).must_equal [2]
     @ds.filter(:flag=>nil).map(:number).must_equal [3]
@@ -560,6 +615,22 @@ describe "Simple Dataset operations" do
     @ds.exclude(:flag=>false).map(:number).must_equal [1, 3]
     @ds.exclude(:flag=>nil).map(:number).must_equal [1, 2]
   end
+
+  cspecify "should deal with boolean equality conditions correctly", :derby do
+    @ds.filter(true=>:flag).map(:number).must_equal [1]
+    @ds.filter(false=>:flag).map(:number).must_equal [2]
+    @ds.filter(nil=>:flag).map(:number).must_equal []
+    @ds.exclude(true=>:flag).map(:number).must_equal [2]
+    @ds.exclude(false=>:flag).map(:number).must_equal [1]
+    @ds.exclude(nil=>:flag).map(:number).must_equal []
+  end
+
+  cspecify "should have exclude_or_null work correctly", :mssql, :derby, :oracle, :db2, :sqlanywhere do
+    @ds = @ds.extension(:exclude_or_null)
+    @ds.exclude_or_null(true=>:flag).map(:number).must_equal [2, 3]
+    @ds.exclude_or_null(false=>:flag).map(:number).must_equal [1, 3]
+    @ds.exclude_or_null(nil=>:flag).map(:number).must_equal [1, 2, 3]
+  end
 end
 
 describe "Simple Dataset operations in transactions" do
@@ -749,8 +820,8 @@ if DB.dataset.supports_cte?
   end
 end
 
-if DB.dataset.supports_cte?(:update) # Assume INSERT and DELETE support as well
-  describe "Common Table Expressions in INSERT/UPDATE/DELETE" do
+if DB.dataset.supports_cte?(:insert) || DB.dataset.supports_cte?(:update) || DB.dataset.supports_cte?(:delete)
+  describe "Common Table Expressions" do
     before do
       @db = DB
       @db.create_table!(:i1){Integer :id}
@@ -763,14 +834,20 @@ if DB.dataset.supports_cte?(:update) # Assume INSERT and DELETE support as well
       @db.drop_table?(:i1)
     end
     
-    it "should give correct results for WITH" do
+    it "should give correct results for WITH in insert" do
       @ds2.insert(@db[:t])
       @ds.select_order_map(:id).must_equal [1, 1, 2, 2]
+    end if DB.dataset.supports_cte?(:insert)
+
+    it "should give correct results for WITH in update" do
       @ds2.filter(:id=>@db[:t].select{max(id)}).update(:id=>Sequel.+(:id, 1))
-      @ds.select_order_map(:id).must_equal [1, 1, 3, 3]
+      @ds.select_order_map(:id).must_equal [1, 3]
+    end if DB.dataset.supports_cte?(:update)
+
+    it "should give correct results for WITH in delete" do
       @ds2.filter(:id=>@db[:t].select{max(id)}).delete
-      @ds.select_order_map(:id).must_equal [1, 1]
-    end
+      @ds.select_order_map(:id).must_equal [1]
+    end if DB.dataset.supports_cte?(:delete)
 
     it "should support a subselect in an subquery used for INSERT" do
       @db.transaction(:rollback=>:always) do
@@ -878,6 +955,8 @@ if DB.dataset.supports_window_functions?
         must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
       @ds.select(:id){sum(:amount).over(:order=>:id).as(:sum)}.all.
         must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>11111, :id=>5}, {:sum=>111111, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:partition=>:group_id, :order=>id.desc, :frame=>:rows).as(:sum)}.all.
+        must_equal [{:sum=>111, :id=>1}, {:sum=>110, :id=>2}, {:sum=>100, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>110000, :id=>5}, {:sum=>100000, :id=>6}]
     end
     
     it "should give correct results for aggregate window functions with frames" do
@@ -891,6 +970,71 @@ if DB.dataset.supports_window_functions?
       @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>:rows).as(:sum)}.all.
         must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>11111, :id=>5}, {:sum=>111111, :id=>6}]
     end
+
+    it "should give correct results for aggregate window functions with ranges" do
+      @ds.select(:id){sum(:amount).over(:partition=>:group_id, :order=>:id, :frame=>:range).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:partition=>:group_id, :order=>:group_id, :frame=>:range).as(:sum)}.all.
+        must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>111000, :id=>6}]
+    end if DB.dataset.supports_window_function_frame_option?(:range)
+
+    it "should give correct results for aggregate window functions with groups" do
+      @ds.select(:id){sum(:amount).over(:partition=>:group_id, :order=>:id, :frame=>:groups).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>:groups).as(:sum)}.all.
+        must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111111, :id=>4}, {:sum=>111111, :id=>5}, {:sum=>111111, :id=>6}]
+    end if DB.dataset.supports_window_function_frame_option?(:groups)
+
+    if DB.dataset.supports_window_function_frame_option?(:offset)
+      it "should give correct results for aggregate window functions with offsets for ROWS" do
+        @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>1}).as(:sum)}.all.
+          must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>110, :id=>3}, {:sum=>1100, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>110000, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>1, :end=>1}).as(:sum)}.all.
+          must_equal [{:sum=>11, :id=>1}, {:sum=>111, :id=>2}, {:sum=>1110, :id=>3}, {:sum=>11100, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>110000, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>2, :end=>[1, :preceding]}).as(:sum)}.all.
+          must_equal [{:sum=>nil, :id=>1}, {:sum=>1, :id=>2}, {:sum=>11, :id=>3}, {:sum=>110, :id=>4}, {:sum=>1100, :id=>5}, {:sum=>11000, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>[1, :following], :end=>2}).as(:sum)}.order(:id).all.
+          must_equal [{:sum=>110, :id=>1}, {:sum=>1100, :id=>2}, {:sum=>11000, :id=>3}, {:sum=>110000, :id=>4}, {:sum=>100000, :id=>5}, {:sum=>nil, :id=>6}]
+      end
+
+      cspecify "should give correct results for aggregate window functions with offsets for RANGES", :mssql, [proc{DB.sqlite_version < 32800}, :sqlite], [proc{DB.server_version < 110000}, :postgres] do
+        @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:range, :start=>1}).as(:sum)}.all.
+          must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111111, :id=>4}, {:sum=>111111, :id=>5}, {:sum=>111111, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:range, :start=>0, :end=>1}).as(:sum)}.all.
+          must_equal [{:sum=>111111, :id=>1}, {:sum=>111111, :id=>2}, {:sum=>111111, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>111000, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:amount, :frame=>{:type=>:range, :start=>100, :end=>1000}).as(:sum)}.all.
+          must_equal [{:sum=>1111, :id=>1}, {:sum=>1111, :id=>2}, {:sum=>1111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>10000, :id=>5}, {:sum=>100000, :id=>6}]
+      end if DB.dataset.supports_window_function_frame_option?(:range)
+
+      it "should give correct results for aggregate window functions with offsets for GROUPS" do
+        @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:groups, :start=>1}).as(:sum)}.all.
+          must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111111, :id=>4}, {:sum=>111111, :id=>5}, {:sum=>111111, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:groups, :start=>0, :end=>1}).as(:sum)}.all.
+          must_equal [{:sum=>111111, :id=>1}, {:sum=>111111, :id=>2}, {:sum=>111111, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>111000, :id=>6}]
+      end if DB.dataset.supports_window_function_frame_option?(:groups)
+    end
+
+    it "should give correct results for aggregate window functions with exclusions" do
+      @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>:preceding, :exclude=>:current}).as(:sum)}.all.
+        must_equal [{:sum=>nil, :id=>1}, {:sum=>1, :id=>2}, {:sum=>11, :id=>3}, {:sum=>111, :id=>4}, {:sum=>1111, :id=>5}, {:sum=>11111, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:rows, :start=>:preceding, :exclude=>:group}).as(:sum)}.all.
+        must_equal [{:sum=>nil, :id=>1}, {:sum=>nil, :id=>2}, {:sum=>nil, :id=>3}, {:sum=>111, :id=>4}, {:sum=>111, :id=>5}, {:sum=>111, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:rows, :start=>:preceding, :exclude=>:ties}).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>10, :id=>2}, {:sum=>100, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>10111, :id=>5}, {:sum=>100111, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>:preceding, :exclude=>:no_others}).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>11111, :id=>5}, {:sum=>111111, :id=>6}]
+    end if DB.dataset.supports_window_function_frame_option?(:exclude)
+
+    it "should give correct results for window functions" do
+      @ds.window(:win, :partition=>:group_id, :order=>:id).select(:id){sum(:amount).over(:window=>win, :frame=>:rows).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
+      @ds.window(:win, :partition=>:group_id).select(:id){sum(:amount).over(:window=>win, :order=>id, :frame=>:rows).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
+      @ds.window(:win, {}).select(:id){sum(:amount).over(:window=>:win, :order=>id, :frame=>:rows).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>11111, :id=>5}, {:sum=>111111, :id=>6}]
+      @ds.window(:win, :partition=>:group_id).select(:id){sum(:amount).over(:window=>:win, :order=>id, :frame=>:all).as(:sum)}.all.
+        must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>111000, :id=>6}]
+    end if DB.dataset.supports_window_clause?
   end
 end
 
@@ -1005,6 +1149,12 @@ describe "Sequel::Dataset#import and #multi_insert :return=>:primary_key " do
     @ds.order(:id).map([:id, :i]).must_equal [[1, 10], [2, 20], [3, 30], [4, 40], [5, 50], [6, 60]]
   end
 
+  it "should handle dataset with row_proc" do
+    ds = @ds.with_row_proc(lambda{|h| Object.new})
+    ds.multi_insert([{:i=>10}, {:i=>20}, {:i=>30}], :return=>:primary_key).must_equal [1, 2, 3]
+    ds.import([:i], [[40], [50], [60]], :return=>:primary_key).must_equal [4, 5, 6]
+  end
+  
   it "should return primary key values when :slice is used" do
     @ds.multi_insert([{:i=>10}, {:i=>20}, {:i=>30}], :return=>:primary_key, :slice=>2).must_equal [1, 2, 3]
     @ds.import([:i], [[40], [50], [60]], :return=>:primary_key, :slice=>2).must_equal [4, 5, 6]
@@ -1332,6 +1482,8 @@ describe "Sequel::Dataset DSL support" do
   
   cspecify "should work with bitwise shift operators", :derby do
     @ds.insert(3, 2)
+    b = Sequel[:b]
+    b = b.cast(:integer) if @db.database_type == :postgres
     @ds.get{a.sql_number << b}.to_i.must_equal 12
     @ds.get{a.sql_number >> b}.to_i.must_equal 0
     @ds.get{a.sql_number << b << 1}.to_i.must_equal 24
@@ -1439,15 +1591,25 @@ describe "Sequel::Dataset DSL support" do
     @ds.exclude(:a=>[20, 10]).all.must_equal []
   end
   
-  it "should work with ranges as hash values" do
+  it "should work with endless ranges as hash values" do
     @ds.insert(20, 10)
-    @ds.filter(:a=>(10..30)).all.must_equal [{:a=>20, :b=>10}]
-    @ds.filter(:a=>(25..30)).all.must_equal []
-    @ds.filter(:a=>(10..15)).all.must_equal []
-    @ds.exclude(:a=>(10..30)).all.must_equal []
-    @ds.exclude(:a=>(25..30)).all.must_equal [{:a=>20, :b=>10}]
-    @ds.exclude(:a=>(10..15)).all.must_equal [{:a=>20, :b=>10}]
-  end
+    @ds.filter(:a=>eval('(30..)')).all.must_equal []
+    @ds.filter(:a=>eval('(20...)')).all.must_equal [{:a=>20, :b=>10}]
+    @ds.filter(:a=>eval('(20..)')).all.must_equal [{:a=>20, :b=>10}]
+    @ds.filter(:a=>eval('(10..)')).all.must_equal [{:a=>20, :b=>10}]
+  end if RUBY_VERSION >= '2.6'
+  
+  it "should work with startless ranges as hash values" do
+    @ds.insert(20, 10)
+    @ds.filter(:a=>eval('(..30)')).all.must_equal [{:a=>20, :b=>10}]
+    @ds.filter(:a=>eval('(...30)')).all.must_equal [{:a=>20, :b=>10}]
+    @ds.filter(:a=>eval('(..20)')).all.must_equal [{:a=>20, :b=>10}]
+    @ds.filter(:a=>eval('(...20)')).all.must_equal []
+    @ds.filter(:a=>eval('(..10)')).all.must_equal []
+    @ds.filter(:a=>eval('(...10)')).all.must_equal []
+
+    @ds.filter(:a=>eval('nil..nil')).all.must_equal [{:a=>20, :b=>10}]
+  end if RUBY_VERSION >= '2.7'
   
   it "should work with CASE statements" do
     @ds.insert(20, 10)
@@ -1672,6 +1834,27 @@ describe "Dataset string methods" do
     @ds.filter(Sequel.expr(:b).ilike(@ds.escape_like('[Bar%]'))).select_order_map(:b).must_equal ['[bar%]']
     @ds.filter(Sequel.expr(:b).ilike("#{@ds.escape_like('Bar%')}_")).select_order_map(:b).must_equal ['bar%.']
     @ds.filter(Sequel.expr(:b).ilike("#{@ds.escape_like('Bar%')}%")).select_order_map(:b).must_equal ['bar%', 'bar%.', 'bar%..']
+
+    Sequel.extension(:escaped_like)
+    @ds.filter(Sequel.expr(:a).escaped_like('?', 'Foo_')).select_order_map(:a).must_equal []
+    @ds.filter(Sequel.expr(:a).escaped_like('?', 'foo_')).select_order_map(:a).must_equal ['foo_']
+    @ds.filter(Sequel.expr(:b).escaped_like('?', ['bar%'])).select_order_map(:b).must_equal ['bar%']
+    @ds.filter(Sequel.expr(:a).escaped_like('??', ['fo', 'o\\_'])).select_order_map(:a).must_equal ['foo\\_']
+    @ds.filter(Sequel.expr(:b).escaped_like('?', 'bar\\%')).select_order_map(:b).must_equal ['bar\\%']
+    @ds.filter(Sequel.expr(:a).escaped_like('?', '[f#*?oo_]')).select_order_map(:a).must_equal ['[f#*?oo_]']
+    @ds.filter(Sequel.expr(:b).escaped_like('?', '[bar%]')).select_order_map(:b).must_equal ['[bar%]']
+    @ds.filter(Sequel.expr(:b).escaped_like('?_', 'bar%')).select_order_map(:b).must_equal ['bar%.']
+    @ds.filter(Sequel.expr(:b).escaped_like('?%', 'bar%')).select_order_map(:b).must_equal ['bar%', 'bar%.', 'bar%..']
+
+    @ds.filter(Sequel.expr(:a).escaped_ilike('?', 'Foo_')).select_order_map(:a).must_equal ['foo_']
+    @ds.filter(Sequel.expr(:a).escaped_ilike('?', 'Foo_')).select_order_map(:a).must_equal ['foo_']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?', ['Bar%'])).select_order_map(:b).must_equal ['bar%']
+    @ds.filter(Sequel.expr(:a).escaped_ilike('??', ['Fo', 'o\\_'])).select_order_map(:a).must_equal ['foo\\_']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?', 'Bar\\%')).select_order_map(:b).must_equal ['bar\\%']
+    @ds.filter(Sequel.expr(:a).escaped_ilike('?', '[F#*?oo_]')).select_order_map(:a).must_equal ['[f#*?oo_]']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?', '[Bar%]')).select_order_map(:b).must_equal ['[bar%]']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?_', 'Bar%')).select_order_map(:b).must_equal ['bar%.']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?%', 'Bar%')).select_order_map(:b).must_equal ['bar%', 'bar%.', 'bar%..']
   end
   
   if DB.dataset.supports_regexp?
